import pandas as pd
import os
from main import *

def getopts(argv):
    opts = {}  # Empty dictionary to store key-value pairs.
    while argv:  # While there are arguments left to parse...
        if argv[0][0] == '-':  # Found a "-name value" pair.
            opts[argv[0]] = argv[1]  # Add key and value to the dictionary.
        argv = argv[1:]  # Reduce the argument list by copying it starting from index 1.
    return opts

def execute():
    from sys import argv
    myargs = getopts(argv)

    if '-i' in myargs:  # Example usage.
        cpout = os.path.abspath(myargs['-i'])
        split = cpout.split(os.path.sep)
        if split[0]=='C:' :
            split[0]='mnt'
            split[1]='mydata'
            join = os.path.join(*split)
            cpout = join
        if cpout[-1] is not '/':
            cpout = cpout + '/'
    else: cpout = '/mnt/mydata/kyuha/desktop/CP_Output/raw_images_1/'
    if '-o' in myargs:
        csvfile = myargs['-o']
    else: csvfile = '/mnt/mydata/kyuha/desktop/hi.csv'
    if '-m' in myargs:
        modeling =  myargs['-m'].strip()
    else: modeling = 'yes'
    if '-n' in myargs:
        clnum = myargs['-n']
    else: clnum = 15

    # cwd = os.path.abspath(os.path.dirname(__file__))
    # cwd = cwd.replace('sourcecode','')
    # direc = textfile.strip()

    if modeling[0] is 'b':
        BuildModel = True
        print 'build'
    else:
        BuildModel = False
        print 'apply'

    firstrun = True

    direc = cpout.strip()
    if direc[-1] is not '/':
        direc = direc + '/'

    main(BuildModel,firstrun,clnum,direc)

    ### below code means nothing, but needed for galaxy format ###
    d = {'dir':[direc]}
    df = pd.DataFrame(data=d)
    df.to_csv(csvfile,index=False, header=False)

    return('analysis completed')



import numpy as np 
from scipy import interpolate
import time

def bd_resample(bd=None,rsnum=None):
    x=bd.transpose()[1]
    y=bd.transpose()[0] 
    if x[-1] != x[0] and y[-1] != y[0]: 
        x=np.append(x,x[0])
        y=np.append(y,y[0])
    sd=np.sqrt(np.power(x[1:]-x[0:-1],2) + np.power(y[1:]-y[0:-1],2))
    sd=np.append([1],sd)
    sid=np.cumsum(sd)
    ss=np.linspace(1,sid[-1],rsnum + 1)
    ss=ss[0:-1] 
    splinerone = interpolate.splrep(sid,x, s=0)
    sx=interpolate.splev(ss,splinerone, der=0)
    splinertwo = interpolate.splrep(sid,y, s=0)
    sy=interpolate.splev(ss,splinertwo, der=0)
    bdrs=np.append([sy],[sx],axis=0)
    return bdrs


#! C:\\Python27
import pandas as pd 
import numpy as np 
import time,re,datetime
#import Tkinter, tkFileDialog
import os

def UI_generator(foldertosort):
	#root = Tkinter.Tk()
	#dirname = tkFileDialog.askdirectory(parent=root,initialdir='/',title='subfolder of CP default output?') + "/"
	#root.quit()
	#dirname = 'C:/Users/Kyu/Desktop/CellProfiler/CP default output folder/'
	dirname = foldertosort
	mastercsv = 'Cells.csv'
	subcsv = 'Image.csv'

	mastercsv = pd.read_csv(dirname + mastercsv)
	no_sets = np.add(range(max(mastercsv.Metadata_SetNumber)),1)

	subcsv = pd.read_csv(dirname + subcsv)
	folderpath = subcsv.PathName_Cells
	
	cell_count = subcsv.Count_Cells.tolist()
	
	idx = pd.factorize(folderpath)[0]
	group_counts = np.empty((len(no_sets),1)).tolist()

	for index, obj in enumerate(cell_count):
		if idx[index]==0:
			group_counts[0].append(obj)
		elif idx[index]==1:
			group_counts[1].append(obj)
	group_counts= np.sum(group_counts,axis=1).astype(int)
	folderpath = np.unique(folderpath)

	foldername = [_ for _ in os.listdir(dirname) if 'set' in _]
	foldername = np.core.defchararray.add(dirname,foldername)

	d = {'date':[datetime.datetime.now()]*len(no_sets),'set number':no_sets, \
	'cell count':group_counts,'raw_imageset_path':folderpath,
	'build_model':np.zeros(len(no_sets)),'apply_model':np.zeros(len(no_sets)), \
	'ctrl/exp?':['NaN']*len(no_sets),'maskset_path':foldername}

	df = pd.DataFrame(data=d)
	df.to_csv(dirname+ 'UI.csv', index=False)

	d = {'date':[],'set number':[], \
	'cell count':[],'raw_imageset_path':[],
	'build_model':[],'apply_model':[], \
	'ctrl/exp?':[],'maskset_path':[]}
	df = pd.DataFrame(data=d)
	masterUIpath = os.path.abspath(os.path.join(foldertosort,'..')) + '/masterUI.csv'
	if not os.path.isfile(masterUIpath):
		df.to_csv(masterUIpath,index=False)



#! C:\Python27
from __future__ import division
from copy import deepcopy
from inspect import getargspec
import time

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib as mpl

from PCA import *

def pca_bdreg_v1(bdpc,VamModel,BuildModel):
    nargin = len(getargspec(pca_bdreg_v1)[0])
    np.set_printoptions(precision=5,suppress=True)

    #bdpc = bdpc.as_matrix()
    # sc = sc.as_matrix()

    Nuu = int(round(len(bdpc.T[0])))
    Nbb = int(round(len(bdpc[0])/2))
    pcnum0 = 12
    dstp = 3
    bdpct = deepcopy(bdpc)

    if BuildModel:
        mmx = np.ones((Nuu,1)) * np.mean(bdpct,axis=0)
    else: 
        mmx = np.ones((Nuu,1)) * VamModel['mdd']   
    smx = np.ones(bdpct.shape)
    test = np.divide((bdpct-mmx),smx)
    if BuildModel:
        pc,score,latent=PCA(test)
        score = np.dot(test,pc)
    else: 
        latent = VamModel['latent']
        pc = VamModel['pc']
        score = np.dot(test,pc)

    mdd = mmx[0]
    sdd = smx[0]
    mss = np.mean(score,axis=0) #score is inaccurate
    sss = np.std(score,axis=0) #score is inaccurate
    cnum = 1

    ce1 = score[cnum-1] #score is inaccurate

    xr = bdpct[cnum-1][0:Nbb]
    yr = bdpct[cnum-1][Nbb:]

    VamModel['mdd']=mdd
    VamModel['sdd']=sdd
    VamModel['pc']=pc
    VamModel['latent']=latent

    plt.figure(1)
    plt.clf()

    #check I: check the shape of principle component; 
    if True:
        plt.figure(22)
        plt.clf()
        for pcnum in range(10):
            dxx = 0.7
            xx = pc.T[pcnum][0:Nbb]
            yy = pc.T[pcnum][Nbb:]
            plt.plot(xx+(pcnum+1)*dxx,yy,'b-',linewidth=2.0)
            plt.plot(-xx+(pcnum+1)*dxx,-yy-dxx,'r-',linewidth=2.0)
        plt.axis('equal')
        plt.axis('off')
        #plt.show()

    #check II: the mean +- std of PC mean
    if True:
        cmap = plt.cm.jet
        vmax = int(11)
        norm = mpl.colors.Normalize(vmin=0,vmax=vmax)
        cid = plt.cm.ScalarMappable(norm=norm,cmap=cmap)

        offx,offy=np.meshgrid(range(pcnum0),[0])
        offx=np.multiply((offx+1),dstp)[0]
        offy=np.multiply(-(offy+1),dstp)[0]

        for kkk in range(pcnum0):
            count=1
            for ks in np.linspace(-10,10,11):
                pnn = np.zeros([len(pc[0])])
                for k in range(Nbb):
                    if k == kkk:
                        pnn=np.add(pnn,pc.T[k]*(mss[k]+ks*sss[k])) #pc[k] has sign issue again
                    else:
                        pnn=pnn+pc.T[k]*mss[k]
                pnn = np.multiply(pnn,sdd.T) + mdd.T
                xx = pnn[0:Nbb]
                yy = pnn[Nbb:]
                xx = np.append(xx,xx[0])
                yy = np.append(yy,yy[0])
                plt.figure(1)
                plt.plot(xx+offx[kkk],yy+offy[kkk],'-',color=cid.to_rgba(count))
                count = count + 1
        plt.axis('equal')
        #plt.show()
    # check III- reconstruct the cell shape using principle
    if True:
        pnn=np.zeros(len(pc[0]))
# pca_bdreg_v1.m:112
        for k in range(pcnum0):
            pnn = pnn + pc.T[k] * ce1[k]
# pca_bdreg_v1.m:114
        pnn=np.multiply(pnn,sdd.T) + mdd.T
# pca_bdreg_v1.m:116
        #       pnn=pnn+mdd';
        xx=pnn[0:Nbb]
# pca_bdreg_v1.m:118
        yy=pnn[Nbb:]
        xx=np.append(xx,xx[0])
        yy=np.append(yy,yy[0])
# pca_bdreg_v1.m:119
        plt.figure(33)
        plt.plot(np.append(xr,xr[0]),np.append(yr,yr[0]),'r-',xx,yy,'b-')
        #bjff3
        plt.axis('equal')
        plt.axis('off')
        #plt.show()

    # check IV - get mean shape of cell
    count=1
    pnn=np.zeros(len(pc[0]))
    for k in range(pcnum0):
        pnn=pnn + np.multiply(pc.T[k],mss[k])
    
    pnn=np.multiply(pnn,sdd.T) + mdd.T
    xx=pnn[0:Nbb]
# pca_bdreg_v1.m:137
    yy=pnn[Nbb:]
# pca_bdreg_v1.m:138
    xx=np.append(xx,xx[0])
# pca_bdreg_v1.m:139
    yy=np.append(yy,yy[0])
# pca_bdreg_v1.m:140
    plt.figure(1311)
    plt.plot(xx,yy,'b-',linewidth=2.0)
    #bjff3
    plt.axis('equal')
    plt.axis('off')

    #plt.show()

    return pc,score,latent,VamModel

def pca_bdreg_main(bdpc,VamModel,BuildModel):
    start = time.time()

    # picklejar='C:/Users/Kyu/Desktop/Vampire Project/Vampire_master/pickle jar/'
    # bdpc = pd.read_pickle(picklejar + 'bdpc.pickle')
    # sc = pd.read_pickle(picklejar + 'sc.pickle')

    pc,score,latent,VamModel=pca_bdreg_v1(bdpc,VamModel,BuildModel)
    end = time.time()
    print 'For PCA, elapsed time is ' + str(end-start) + 'seconds...'
    return pc, score, latent, VamModel
'''
    df1 = pd.DataFrame(pc)
    df2 = pd.DataFrame(score)
    df3 = pd.DataFrame(latent)

    df1.to_pickle(picklejar + 'pc.pickle')
    df2.to_pickle(picklejar + 'score.pickle')
    df3.to_pickle(picklejar + 'latent.pickle') 
'''


#! C:\Python27
from copy import deepcopy
# from inspect import getargspec
import time
import os

import numpy as np
import pandas as pd

from bd_resample import *
from reg_bd_svd import *
from reg_bd3 import *

def bdreg_v1(B,N=None,VamModel=None,BuildModel=None):
    np.set_printoptions(precision=5,suppress=True)

    if N == None:
        N=50

    if not BuildModel:
        print 'applying model'
        N = VamModel['N']
    elif BuildModel:
        print 'building model'
        VamModel['N']= N

    bduse=deepcopy(B)
    plotres=0
    bnreg=deepcopy(bduse)
    bnreg0=deepcopy(bnreg)
    kll=len(bduse)
    bdpc=np.zeros([kll,2*N])
    bdpc0=deepcopy(bdpc)
    sc=np.zeros([kll,1])
    for ktt in range(kll):
        bdt=bd_resample((bduse.loc[ktt]),N)
        bnreg0.loc[ktt],sc[ktt]=reg_bd_svd(bdt)
        bdpc0[ktt]=np.append([bnreg0[ktt][1]],[bnreg0[ktt][0]],axis=1)
    mbdpc0 = [sum(x)/len(x) for x in zip(*bdpc0)]
    bdr0=np.append([mbdpc0[N:]],[mbdpc0[0:N]],axis=0)

    if BuildModel:
        bdrn=deepcopy(bdr0)
        VamModel['bdrn']=bdrn
    else:
        bdrn=VamModel['bdrn']

    outyt=np.zeros([kll,1])
    bnreg=deepcopy(bnreg0)

    for tt0 in range(1):
        for ktt in range(kll):
            bnreg[ktt],outyt[ktt]=reg_bd3(bnreg.loc[ktt],bdrn)
            bdpc[ktt]=np.append(bnreg[ktt][1],bnreg[ktt][0])
            if plotres: #no plotting
                print(ytot)
                bdnew=bnreg.loc[ktt]
                figure(2)
                plot(bdnew[:,2],bdnew[:,1],'b-',bdr0[:,2],bdr0[:,1],'r-')
                hold('on')
                plot(bdnew[1,2],bdnew[1,1],'ro')
                plot(bdnew[5,2],bdnew[5,1],'go')
                plot(bdnew[10,2],bdnew[10,1],'mo')
                plot(bdr0[1,2],bdr0[1,1],'ro')
                plot(bdr0[5,2],bdr0[5,1],'go')
                plot(bdr0[10,2],bdr0[10,1],'mo')
                hold('off')
                axis('square')
                axis('equal')
        mbdpc=np.mean(bdpc,axis=0)
        bdrn=np.append([mbdpc[N:]],[mbdpc[0:N]],axis=0) # revise pt 5
    return bdpc, bnreg, sc, VamModel

def bdreg_main(df,N=None,VamModel=None,BuildModel=None):
    start = time.time() #record time
    bdpc, bnreg, sc, VamModel=bdreg_v1(df[0],N,VamModel,BuildModel)
    end = time.time()
    print 'For bdreg, elapsed time is ' + str(end-start) + 'seconds...'

    picklejar = os.path.abspath(os.path.dirname(__file__))
    picklejar = picklejar.replace('sourcecode','picklejar') + '/'
    df1 = pd.DataFrame(bdpc)
    df2 = pd.DataFrame(bnreg)
    df3 = pd.DataFrame(sc)
    df1.to_pickle(picklejar + 'bdpc.pickle')
    df2.to_pickle(picklejar + 'bnreg.pickle')
    df3.to_pickle(picklejar + 'sc.pickle')

    return bdpc, bnreg, sc, VamModel



'''
# bdreg_v1.m:79
        if plotres: #skip this for now
            figure(990)
            plot(bdrn[:,2],bdrn[:,1],'b-')
            hold('on')
            disp(mean(min(outyt,[],1)))
    if plotres: #skip this for now
        # show aligned image from random shuffle objects
        idr=rand(kll,1)
# bdreg_v1.m:90
        __,sidr=sort(idr,nargout=2)
# bdreg_v1.m:91
        for k in arange(1,30).reshape(-1):
            ktt=sidr[k]
# bdreg_v1.m:94
            bdd=bnreg[ktt]
# bdreg_v1.m:95
            bdd=matlabarray(cat(bdd[:,1],bdd[:,2]))
# bdreg_v1.m:96
            figure(88)
            plot(bdd[cat(arange(1,end()),1),2],bdd[cat(arange(1,end()),1),1],'b')
            hold('on')
'''



def PCA(data, dims_rescaled_data=100):
    """
    returns: data transformed in 2 dims/columns + regenerated original data
    pass in: data as 2D NumPy array
    """
    import numpy as NP
    from scipy import linalg as LA
    m, n = data.shape
    # mean center the data
    data -= data.mean(axis=0)
    # calculate the covariance matrix
    R = NP.cov(data, rowvar=False)
    # calculate eigenvectors & eigenvalues of the covariance matrix
    # use 'eigh' rather than 'eig' since R is symmetric, 
    # the performance gain is substantial
    evals, evecs = LA.eigh(R)
    # sort eigenvalue in decreasing order
    idx = NP.argsort(evals)[::-1]
    evecs = evecs[:,idx]
    # sort eigenvectors according to same index
    evals = evals[idx]
    # select the first n eigenvectors (n is desired dimension
    # of rescaled data array, or dims_rescaled_data)
    evecs = evecs[:, :dims_rescaled_data]
    # carry out the transformation on the data using eigenvectors
    # and return the re-scaled data, eigenvalues, and eigenvectors
    return evecs,NP.dot(evecs.T, data.T).T,evals


#! C:\Python27
import os
import math
import numpy as np
import pandas as pd
import cv2
import time
'''
Read binary mask of cells generated by the CellProfiler
Find coordinates of the cell outline
Organize coordinates in a dataframe
Save dataframe as a pickle to pass it to another python script
'''

#print upto 4 degits below zero
np.set_printoptions(precision=4,suppress=True)

def read_selected_imageset(masterUIpath,BuildModel=False):
	#cwd = os.path.abspath(os.path.dirname(__file__))
	#UI = pd.read_csv(cwd + '/' + 'masterUI.csv')

	UI = pd.read_csv(masterUIpath + 'masterUI.csv')

	if BuildModel:
		on = UI.build_model
		tag = 'build_model'
	else:
		on = UI.apply_model
		tag = 'apply_model'

	setpath = UI.maskset_path
	on = [0 if math.isnan(x) else x for x in on]
	index = np.argwhere(on).flatten()

	activeset = setpath[index]

	print 'converting masks from ........'
	print activeset

	cellphotos=[]
	for index, dirname in enumerate(activeset):
		dirname = dirname + '/'
		imagefolders = os.listdir(dirname)
		for folder in imagefolders:
			subdir = dirname + folder +'/'
			cellphoto = os.listdir(subdir)
			cellphoto = np.core.defchararray.add(subdir,cellphoto)
			cellphotos = np.append(cellphotos, cellphoto)
	#iterate each cell photo to record pixel coordinate into dataframe
	boundarymaster=[]
	for cellbinary in cellphotos:
		#load image
		im = cv2.imread(cellbinary)
		#find boundary using openCV function
		imgray = cv2.cvtColor(im,cv2.COLOR_BGR2GRAY)
		ret,thresh = cv2.threshold(imgray,127,255,0)
		im2, contours, hierarchy = cv2.findContours(thresh,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
		#postprocess boundary to match with matlab
		contour = np.empty(2)
		for i in range(len(contours[0])):
			contour = np.vstack((contours[0][i][0],contour))
		contour=contour[0:-1]
		contour.T[0] = contour.T[0]+1
		contour.T[1] = contour.T[1]+1

		boundary = np.empty((2,len(contour.T[1])))
		boundary[0] = contour.T[1]
		boundary[1] = contour.T[0]
		boundary = boundary.T.astype(int)
		#record
		boundarymaster.append(boundary)

	df = pd.DataFrame(boundarymaster)
	### need to edit this specific path to relative path
	#cwd = os.path.abspath(os.path.dirname(__file__))
	#dst = cwd.replace('sourcecode','picklejar')
	#dst = dst.replace('\\','/')
	#dst = dst + '/'

	dst = masterUIpath + 'picklejar/'
	path = dst + tag + 'Vampire_Input' +'.pickle'

	if not os.path.exists(dst):
		os.makedirs(dst)
	df.to_pickle(path)
	return df



#! C:\Python27
import os
import re
import time
import pandas as pd
import pickle
import random
from maskreader import read_selected_imageset
from bdreg import *
from pca_bdreg import *
from clusterSM import *


def recordIDX(IDX,BuildModel,cpout):
	#cwd = os.path.abspath(os.path.dirname(__file__))
	#UI = pd.read_csv(cwd + '/' + 'masterUI.csv')
	UI = pd.read_csv(cpout + 'masterUI.csv')

	if BuildModel:
		on = UI.build_model
		tag = 'build_model'
	else:
		on = UI.apply_model
		tag = 'apply_model'

	setpath = UI.maskset_path
	index = np.argwhere(on).flatten()

	activeset = setpath[index].tolist()
	activefolder = os.path.dirname(activeset[0])

	CellsCSV = pd.read_csv(activefolder + '/Cells.csv')

	idx = np.argwhere(np.isnan(CellsCSV.Location_Center_X))
	idx = idx.flatten()

	if np.isnan(CellsCSV.Location_Center_X.tolist()[-1]):
		IDX = np.append(IDX,np.nan)
	IDX = np.insert(IDX,idx,np.nan)
	IDX = np.delete(IDX,-1)

	CellsCSV['IDX']=IDX
	CellsCSV.to_csv(activefolder + '/Cells.csv')

def main(BuildModel,firstrun,clnum,cpout):
	if BuildModel:
		if firstrun == True:
			df = read_selected_imageset(cpout,BuildModel) #convert mask to pickle for choosen image set

		#picklejar = os.path.abspath(os.path.dirname(__file__))
		#picklejar = picklejar.replace('sourcecode','picklejar')
		picklejar = cpout + 'picklejar/'

		apply_input = [_ for _ in os.listdir(picklejar) if 'build_model' in _]
		df = pd.read_pickle(picklejar +'/'+ apply_input[0])

		VamModel = {
		"N":[],
		"bdrn":[],
		"mdd":[],
		"sdd":[],
		"pc":[],
		"latent":[],
		"clnum":[],
		"pcnum":[],
		"mincms":[],
		"testmean":[],
		"teststd":[],
		"boxcoxlambda":[],
		"C":[],
		"dendidx":[]
		}

		N = None
		bdpc, bnreg, sc, VamModel = bdreg_main(df,N,VamModel,BuildModel)
		pc , score, latent, VamModel = pca_bdreg_main(bdpc,VamModel,BuildModel)
		pcnum=None

		IDX,bdsubtype,C,VamModel = cluster_main(score,pc,bdpc,clnum,pcnum,VamModel,BuildModel)
		if os.path.exists(picklejar + 'VamModel.pickle'):
			f=open(picklejar + 'VamModel_' + str(random.randint(0,100)) +'.pickle','wb')
			pickle.dump(VamModel,f)
			f.close()
		else:
			f=open(picklejar + 'VamModel.pickle','wb')
			pickle.dump(VamModel,f)
			f.close()

		print 'Model Saved'

		result = recordIDX(IDX,BuildModel,cpout)

	else:
		df = read_selected_imageset(cpout,BuildModel)

		# picklejar = os.path.abspath(os.path.dirname(__file__))
		# picklejar = picklejar.replace('sourcecode','picklejar') +'/'
		picklejar = cpout + 'picklejar/'
		
		f=open(picklejar + 'VamModel.pickle','r')
		VamModel = pickle.load(f)

		apply_input = [_ for _ in os.listdir(picklejar) if 'apply_model' in _]
		df = pd.read_pickle(picklejar + apply_input[0])

		N = VamModel['N']
		#input bdpc,score,pc in new formats
		bdpc_new, bnreg_new, sc_new, VamModel = bdreg_main(df,N,VamModel,BuildModel)
		pc_new, score_new, latent_new, VamModel = pca_bdreg_main(bdpc_new,VamModel,BuildModel)

		clnum=VamModel['clnum']
		pcnum=VamModel['pcnum']
		#pc_new goes in for sake of placing, but pc from the model is used in cluster_main
		IDX_new,bdsubtype_new,C_new,VamModel = cluster_main(score_new,pc_new,bdpc_new,clnum,pcnum,VamModel,BuildModel)
		result = recordIDX(IDX_new,BuildModel)
		# build = [bdpc,bdsubtype,bnreg,C,IDX,latent,pc,sc,score]
		# new = [bdpc_new,bdsubtype_new,bnreg_new,C_new,IDX_new,latent_new,pc_new,sc_new,score_new]

		# f=open('build.pickle','wb')
		# pickle.dump(build,f)
		# f.close()

		# f=open('new.pickle','wb')
		# pickle.dump(new,f)
		# f.close()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="vampireingalaxy",
    version="0.0.18",
    author="Kyu Sang Han",
    author_email="khan21@jhu.edu",
    description="Vampire Image Analysis Package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://wirtzlab.johnshopkins.edu",
    packages=setuptools.find_packages(),
    install_requires=[
        'scipy',
        'pandas',
        'numpy',
        'pillow',
        'matplotlib',
        'scikit-learn',
        'imageio',
        #'opencv'
    ],
    scripts=['bin/vampire-run.py','bin/sort-run.py','bin/analysis-run.py'],
    classifiers=(
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
    ),
)


#! C:\\Python27

import os
import time
import re

import pandas as pd
import numpy as np
import imageio
from scipy import misc
################################
# How this script works #
# 1. read binary masks <- output of segmentation by cell profiler
# 2. compare with centroid recorded in a csv file
# 3. label the masks to the corresponding original image
# 4. organize masks into a subfolder named after original image
################################
# Detailed explanation #
# Reading entire set of mask is heavywork, we first compare object #1 with centroid #1 of set 1,2,...n.
# note: comparing means see if mask has light intensity at the centroid
# If we are "unlucky", there could be two masks from two different sets overlap, then we compare object #2,3,...
# the timestamp for the same set varies as hundreds of mask cannot be generated simultaneously
# this is trouble in labeling masks that have different timestamp then the masks compared with centroid
# I used clustering of timestamp in addition to labeling the first timestamp of each cluster
# Thus, robust classification is achieved....
################################
# Assumptions #
# 1. csv name is always Cells.csv  Note: csv can be named in cell profiler, we assure the user not to touch this in cell profiler.
# 2. user knows where binary files are stored
# 3. the number of objects in a raw image is less than 9999
################################
def read_location(csv,objectnum):
	x = csv.Location_Center_X
	y = csv.Location_Center_Y
	objnum = csv.ObjectNumber
	index = np.where(objnum == objectnum+1)[0] #first object location
	x = x[index].fillna(0).astype(int).as_matrix()
	y = y[index].fillna(0).astype(int).as_matrix()
	return x,y

def read_nth_object_mask(dirname,masknames):
	mask_matrix = []
	for mask in masknames:
		im = misc.imread(dirname + mask)
		#im = imageio.imread(dirname + mask,format=None)
		mask_matrix.append(im)
	return mask_matrix

def check_location_in_mask(locx,locy,mask_matrix,masknames):
	imagenum=[]
	corresmask=[]
	for i in range(len(locx)):
		matched_mask=[]
		x= locx[i]
		y= locy[i]

		for j in range(len(mask_matrix)):
			maskinuse = np.divide(mask_matrix[j],255)
			intensity = maskinuse[y][x]
			if intensity == 1:
				matched_mask.append(masknames[j])

		if len(matched_mask) == 0:
			locresult = None
			status = 'inconclusive'
		elif len(matched_mask) == 1:
			#register mask name in matched_mask [x.tiff,y.tiff,z.tiff]
			corresmask.append(matched_mask)
			#register order in image num [1,2,3,4,...]
			imagenum.append([i+1])
			#put index and maskname in a 2d array
			locresult = np.append([imagenum],[corresmask],axis=0)
			status = 'conclusive'
		elif len(matched_mask) > 1:
			locresult = None
			status = 'inconclusive'
		
	return locresult, status

def sortimage(res,dirname,directory):#resminus,res0,res1,res2,res3
	unicodelist = os.listdir(dirname)
	stringlist = [x.encode('UTF8') for x in unicodelist]
	for i in range(len(res)):
		movelist = [_ for _ in stringlist if res[i] in _]
		for filename in movelist:
			current = dirname + filename
			destination = directory + filename
			os.rename(current,destination)

def sortset(no_objects_per_set,dirname):
	foldername = [_ for _ in os.listdir(dirname) if 'image' in _]
	for idx,obj in enumerate(no_objects_per_set):
		dst  = dirname+ 'set ' + str(idx+1) +'/'
		if not os.path.isdir(dst):
			os.makedirs(dst)
		for i in range(obj):
			current = dirname + foldername[0]
			destination = dst + foldername[0]
			os.rename(current,destination)
			del foldername[0]

def grouper(iterable,startingnum):
	prev = None
	group = []
	cut = 1
	for item in iterable:
		if not prev or item - prev <= 1 and item < startingnum[cut]:
			group.append(item)
		else:
			if cut < len(startingnum):
				cut = cut + 1
			yield group
			group = [item]
		prev = item
	if group:
		yield group

def clusterer(dirname,no_images,locresult):
	masks = [_ for _ in os.listdir(dirname) if '.tiff' in _]
	numbers=set()
	for idx,obj in enumerate(masks):
		res = re.findall("_([0-9]{10}).tiff",obj)[0]
		numbers.add(res)
	numbers = list(numbers)
	numbers = [x.encode('UTF8') for x in numbers]
	numbers = np.sort(np.asarray(numbers).astype(int))

	startingnum = []
	for i in range(no_images):
		fullname = locresult[1][i][0]
		res = re.findall("_([0-9]{10}).tiff",fullname)[0]
		res = int(res)
		startingnum.append(res)
	startingnum.append(9999999999)
	startingnum = np.sort(startingnum)
	cluster = dict(enumerate(grouper(numbers,startingnum), 1))
	if len(cluster) != no_images:
		#sort error
		return None
	return cluster


##### check first mask with centroid, and then if there are two masks overlapping, move on to object #2,3,..

def masksort(dirname):
	csv = pd.read_csv(dirname + 'Cells.csv') #later replace filename automatically
	longest_obj = max(csv.ObjectNumber)
	setnum = csv.Metadata_SetNumber
	no_sets = max(setnum)
	imagenum = csv.ImageNumber
	no_images = max(imagenum)
	no_objects_per_set = []
	for sets in range(no_sets):
		index = np.argwhere(setnum == sets+1).flatten()
		if sets == 0 :
			no_objects_in_a_set = max(imagenum[index])
		else: no_objects_in_a_set = max(imagenum[index]) - no_objects_per_set[sets-1]
		no_objects_per_set.append(no_objects_in_a_set)

	#for loop here
	for objectnum in range(longest_obj):
		print 'iteration' + str(objectnum + 1) + '/' + str(longest_obj)
		masknames = []
		masklist = [_ for _ in os.listdir(dirname) if str(objectnum+1) + "_" in _]
		if objectnum < 9:
			for fname in masklist:
				res = re.search("_([0]{3}[0-9])_",fname) #find two digit names
				if not res==None: masknames.append(fname)
		if objectnum > 8 and objectnum <99:
			for fname in masklist:
				res = re.search("_[0]{2}([0-9]{2})_",fname) #find two digit names
				if not res==None: masknames.append(fname)
		if objectnum > 98 and objectnum <999:
			for fname in masklist:
				res = re.search("_[0]([0-9]{3})_",fname) #find two digit names
				if not res==None: masknames.append(fname)
		if objectnum > 998:
			for fname in masklist:
				res = re.search("_([0-9]{4})_",fname) #find two digit names
				if not res==None: masknames.append(fname)
		mask_matrix = read_nth_object_mask(dirname,masknames) #read masks in directory as 3d arrays
		locx,locy = read_location(csv,objectnum) #read location of first masks of each image
		locresult,status = check_location_in_mask(locx,locy,mask_matrix,masknames) #compare location with each mask and determine corresponding mask
		#locresult is a list of pair of objectnum and mask filename

		if status == 'inconclusive': continue
		elif status == 'conclusive':
			print 'saving after ' + str(objectnum+1) + 'th object verification'
			#clustering timestamp
			cluster = clusterer(dirname,no_images,locresult)
			if cluster == None:
				print 'sort error!'
				return()
			#save
			for i in range(no_images): #repeat 4 times now
				directory = dirname +'image' + str(i+1) +'/'
				if not os.path.exists(directory):
					os.makedirs(directory)
				fullname = locresult[1][i][0]
				#similar time grouping <--- this need to be replaced with better method: check unix time excel export from cell profiler
				res = re.findall("_([0-9]{10}).tiff",fullname)[0]
				res = int(res) #convert res from unicode to int to compare with cluster of int
				#cluster was converted into int to sort the timestamp
				res = [x for x in cluster.values() if res in x][0]
				res = [str(x) for x in res]
				sortimage(res,dirname,directory)
			sortset(no_objects_per_set,dirname)
			return()

#start=time.time()
#main()
#end=time.time()
#print 'elapsed time is ' + str(end-start) + 'seconds...'



from __future__ import division

from copy import deepcopy
from inspect import getargspec
import time

import pandas as pd
import numpy as np
from scipy import stats, cluster, spatial

from sklearn.cluster import KMeans
from sklearn import preprocessing

import matplotlib.pyplot as plt
import matplotlib as mpl
from matplotlib.ticker import NullFormatter

from PCA import *

def clusterSM(score,pc,bdpc,clnum=None,pcnum=None,VamModel=None,BuildModel=None):
    np.set_printoptions(precision=4,suppress=True)

    Nuu=int(round(len(bdpc.T[0])))
    Nbb=int(round(len(bdpc[0]) / 2))
    mmx=np.dot(np.ones([Nuu,1]),np.mean([bdpc],axis=1))
    smx=np.ones(bdpc.shape)
    mdd=mmx[0]
    sdd=smx[0]
    mss=np.mean(score,axis=0)
    sss=np.std(score,axis=0)

    NN=10

    if clnum==None:
        clnum=15
    if pcnum==None:
        pcnum=20

    if BuildModel:
        VamModel['clnum']=clnum
        VamModel['pcnum']=pcnum
    else:
        clnum = VamModel['clnum']
        pcnum = VamModel['pcnum']
        pc = VamModel['pc']

    cms00 = score[:,0:pcnum]
    cms = deepcopy(cms00)

    if BuildModel:
        mincms=np.amin(cms,axis=0)
        VamModel['mincms']=mincms
        VamModel['boxcoxlambda']=np.zeros(len(cms.T))
        VamModel['testmean']=np.zeros(len(cms.T))
        VamModel['teststd']=np.zeros(len(cms.T))
    else:
        mincms=VamModel['mincms']

    for k in range(len(cms.T)):
        test=cms.T[k]
        test=test - mincms[k] + 1
        if BuildModel:
            test,maxlog=stats.boxcox(test)
            test = np.asarray(test)
            #####################
            VamModel['boxcoxlambda'][k]=maxlog
            VamModel['testmean'][k]=np.mean(test)
            VamModel['teststd'][k]=np.std(test)
            #####################
            cms.T[k]=(test-np.mean(test))/np.std(test)
        else:
            test=stats.boxcox(test,VamModel['boxcoxlambda'][k])
            cms.T[k]=(test-VamModel['testmean'][k])/VamModel['teststd'][k]
    cms0=deepcopy(cms)

    reduce_dim='none'
    if ('cluster') == (reduce_dim):
        cmst=deepcopy(cms.T)
        cmst_Norm = preprocessing.normalize(cmst)
        kmeans=KMeans(n_clusters=clnum, n_init=5).fit(cmst_Norm)
        IDXp = kmeans.labels_
        sid=np.argsort(IDXp)
        cmsts=cmst[sid]
        cmsn=np.zeros([len(cms.T[0]),clnum])
        for k in range(clnum):
            cck= IDXp == k #cck is logical matrix
            ptemp=np.mean(cmst[cck],axis=0)
            cmsn.T[k]=ptemp
    elif ('none') == (reduce_dim):
            cmsn=deepcopy(cms)
    rowdist='cosine'
    coldist='Euclidean'
    link='complete'
    method='kmean'

    if ('dend') == (method):
        Z=cluster.hierarchy.linkage(cmsn,method='complete',metric='cosine')
        IDX=cluster.hierarchy.fcluster(Z,t=clnum,criterion='maxclust') #Z is the same but IDX is not....
        th=np.multiply(0,max(Z.T[2]))
    elif ('kmean') == (method):
        if BuildModel:
            cmsn_Norm = preprocessing.normalize(cmsn)

            #make clnum str -> int if str
            if isinstance(clnum, basestring):
                clnum = int(clnum)

            kmeans=KMeans(n_clusters=clnum,init='k-means++',n_init=3,max_iter=300).fit(cmsn_Norm) #init is plus,but orginally cluster, not available in sklearn
            IDX = kmeans.labels_
            C = kmeans.cluster_centers_
            ##################
            VamModel['C']=C
            D = spatial.distance.cdist(cmsn,VamModel['C'])
            IDX = np.argmin(D,axis=1)
        else:
            C = VamModel['C']
            D = spatial.distance.cdist(cmsn,VamModel['C'])
            IDX = np.argmin(D,axis=1)
            ##################

    clnum=max(IDX)+1
    offx,offy=np.meshgrid(range(clnum),[0])
    offx=np.multiply(offx,1)+1
    offx=offx[0]*1-0.5
    offy=np.subtract(np.multiply(offy,1), 1.5)+1
    offy=offy[0]
    cmap = plt.cm.jet
    vmax = int(clnum*10)
    norm = mpl.colors.Normalize(vmin=0,vmax=vmax)
    cid = plt.cm.ScalarMappable(norm=norm,cmap=cmap)

    clshape=np.zeros([clnum,len(cms00[0])])
    clshapesdv=deepcopy(clshape)
    bdst0=np.empty(len(bdpc.T))
    bdst=deepcopy(bdst0)
    for kss in range(clnum):
        c88=IDX == kss
        bdpcs=bdpc[c88,:]
        mbd=np.mean(bdpcs,axis=0)
        bdst0=np.vstack((bdst0,mbd))
    bdst0=bdst0[1:]
    # dendrogram of the difference between different shape
    if BuildModel:
        Y=spatial.distance.pdist(bdst0,'euclidean')
        Z=cluster.hierarchy.linkage(Y,method='complete') #4th row is not in matlab
        Z[:,2] = Z[:,2]*10 #multiply distance manually 10times to plot better.
        cluster.hierarchy.set_link_color_palette(['m', 'c', 'y', 'k'])
        fig = plt.figure(289,figsize=(6,2),linewidth=2.0)
        R=cluster.hierarchy.dendrogram(Z,p=0,truncate_mode='mlab',orientation='bottom',ax=None)
        leaflabel = np.array(R['ivl'])
        dendidx = leaflabel
        VamModel['dendidx']=dendidx
        cluster.hierarchy.set_link_color_palette(None)
    else:
        dendidx = VamModel['dendidx']
    plt.axis('equal')
    plt.axis('off')
    #h=get(gca,'children')  didn activate this feature yet
    IDXsort=np.zeros(len(IDX))
    for kss in range(clnum):
        c88=IDX == int(dendidx[kss])
        IDXsort[c88]=kss
    IDX=deepcopy(IDXsort)
    bdsubtype=np.empty((int(max(IDX)+1),2,Nbb+1)) #need more specific preallocation: 2 for x and y, Nbb+1 for len(x)
    for kss in range(int(max(IDX))+1):
        c88=IDXsort == kss
        clshape[kss]=np.mean(cms00[c88],axis=0)
        clshapesdv[kss]=np.std(cms00[c88],axis=0)
        pnn=np.zeros(len(pc.T[0]))
        for kev in range(len(cms00[0])):
            pnn=np.add(pnn,np.multiply(pc.T[kev],clshape[kss,kev]))
            pnnlb=np.add(pnn,np.multiply(np.multiply(pc.T[kev],-2),clshapesdv[kss,kev]))
            pnnhb=np.add(pnn,np.multiply(np.multiply(pc.T[kev],2),clshapesdv[kss,kev]))
        pnn=np.multiply(pnn,sdd) + mdd
        pnnlb=np.multiply(pnnlb,sdd) + mdd
        pnnhb=np.multiply(pnnhb,sdd) + mdd  #pnn,pnnlb&hb are all randomized
        xx=pnn[0:Nbb]
        yy=pnn[Nbb:]
        xlb=pnnlb[0:Nbb]
        ylb=pnnlb[Nbb:]
        xhb=pnnhb[0:Nbb]
        yhb=pnnhb[Nbb:]
        xx=np.append(xx,xx[0])
        yy=np.append(yy,yy[0])
        xxhb=np.append(xhb,xhb[0])
        yyhb=np.append(yhb,yhb[0])
        xxlb=np.append(xlb,xlb[0])
        yylb=np.append(ylb,ylb[0])
        fss=4
        plt.figure(289)
        plt.axis('equal')
        plt.plot((xx/fss+offx.T[kss])*10, (yy/fss+offy.T[kss])*10, '-',color=cid.to_rgba(kss)) #this is not plotted in matlab as well
        bdsubtype[kss][0]=xx/fss
        bdsubtype[kss][1]=yy/fss

        bdpcs=bdpc[c88]
        mbd=np.mean(bdpcs,axis=0)
        bdNUM=int(round(len(mbd)/2))
        bdst=np.vstack((bdst,mbd))
        xaxis = np.add(np.divide(np.append(mbd[0:bdNUM],mbd[0]),fss),offx[kss])*10
        yaxis = np.add(np.divide(np.append(mbd[bdNUM:],mbd[bdNUM]),fss),offy[kss])*10
        plt.plot(xaxis,yaxis,'-')
        sid=np.argsort(np.random.rand(sum(c88),1),axis=0)

        if len(sid)<NN: enum = len(sid)
        else:enum = NN
        for knn in range(enum):
            plt.figure(922)
            x99=bdpcs[sid[knn],np.append(range(bdNUM),0)]
            y99=bdpcs[sid[knn],np.append(np.arange(bdNUM,(bdNUM*2),1),bdNUM)]
            xax = np.add(np.divide(x99,fss),offx[kss])
            yax = np.add(np.divide(y99,fss),offy[kss])
            plt.plot(xax,yax,'r-',linewidth=0.1)
    bdst = bdst[1:]

    # result visulation
    plt.figure(922)
    plt.axis('equal')
    plt.axis('off')
    #plt.close()
    #axis('ij')
    plt.figure(289)
    plt.axis('equal')
    plt.axis('off')
    #axis('ij')
    ###
    crap=np.sort(IDX)
    sid=np.argsort(IDX)
    BMsort=cmsn[sid]
    ### show the before and after sorted result
    plt.figure(20)
    plt.imshow(cmsn,extent=[0,1,0,1])
    #set(gca,'clim',cat(- 1,1))
    plt.clim(-1,1)
    #colormap(cmapW(100))
    #bjff3
    #set(gca,'xtick',cat(arange(1,length(cms[1,:]))),'xticklabel',[])
    #plt.xticks(np.arange(len(cms[0])))
    plt.gca().xaxis.set_minor_formatter(NullFormatter())
    #set(gca,'tickdir','out')
    #set(gca,'ytick',[],'yticklabel',[])
    plt.yticks()
    plt.gca().yaxis.set_minor_formatter(NullFormatter())
    # figure(21);imagesc(BMsort);
    #plt.close()

    plt.figure(21)
    plt.imshow(BMsort,extent=[0,1,0,1]) #imagesc(cat(BMsort))

    #set(gca,'clim',cat(- 1,1)) #color limits
    #colormap(cmapW(100)) what is cmapW ??
    #bjff3
    #set(gca,'xtick',cat(arange(1,length(cms[1,:]))),'xticklabel',[])
    #plt.xticks(np.arange(len(cms[0])))
    #set(gca,'tickdir','out')
    plt.gca().tick_params(direction='out')
    #set(gca,'ytick',[],'yticklabel',[])
    plt.yticks()
    plt.gca().yaxis.set_minor_formatter(NullFormatter())
    ### show the scale bar
    #plt.close()

    plt.figure(23)
    crap = np.expand_dims(crap,axis=1)
    imgplot = plt.imshow(crap,cmap='jet',extent=[0,1,0,1]) #imagesc(crap)
    #bjff3
    plt.axis('on')
    #set(gca,'xtick',[],'xticklabel',[])
    plt.xticks()
    plt.gca().xaxis.set_minor_formatter(NullFormatter())
    #set(gca,'ytick',[],'yticklabel',[])
    plt.yticks()
    plt.gca().yaxis.set_minor_formatter(NullFormatter())
    #plt.close()

    plt.figure(22)
    #count,bin=hist(IDX,range(clnum+1))
    n,bins,patches = plt.hist(IDX,range(clnum+1))
    plt.bar(np.delete(bins,0),n)
# clusterSM.m:237
    #bjff3
    #xlim(cat(0,clnum + 1))
    #plt.xlim( (-1,clnum+1) )
    #set(gca,'tickdir','out')
    plt.gca().tick_params(direction='out')
    #set(gca,'xtick',cat(arange(1,clnum)))
    #plt.xticks(np.arange(clnum+1))
    #set(gca,'xticklabel',[])
    plt.gca().xaxis.set_minor_formatter(NullFormatter())
    #set(gcf,'position',cat(552,434,440,186))
    #plt.close()


    #plt.show()


    return IDX,bdsubtype,C,VamModel

def cluster_main(score,pc,bdpc,clnum=None,pcnum=None,VamModel=None,BuildModel=None):

    start = time.time()

    # picklejar='C:/Users/Kyu/Desktop/Vampire Project/Vampire_master/pickle jar/'

    # score = pd.read_pickle(picklejar + 'score.pickle')
    # pc = pd.read_pickle(picklejar + 'pc.pickle')
    # bdpc = pd.read_pickle(picklejar + 'bdpc.pickle')

    # bdpc = bdpc.as_matrix()
    # score = score.as_matrix()
    # pc = pc.as_matrix()

    IDX,bdsubtype,C,VamModel=clusterSM(score,pc,bdpc,clnum,pcnum,VamModel,BuildModel)
    end = time.time()
    print 'For cluster, elapsed time is ' + str(end-start) + 'seconds...'
    return IDX,bdsubtype,C,VamModel
    # df1 = pd.DataFrame(IDX)
    # df2 = pd.DataFrame(C)

    # df1.to_pickle(picklejar + 'IDX.pickle')
    # df2.to_pickle(picklejar + 'C.pickle')


#bdsubtype cannot be exported as it is 3d array that is similar to cell array of MATLAB



import os
textfile = os.path.abspath('C:/Users/kyuha/Desktop/CP_Output/raw_images_1')
split = textfile.split(os.path.sep)
if split[0]=='C:' :
	split[0]='mnt'
	split[1]='mydata'
	join = os.path.join(*split)
	textfile = join
if textfile[-1] is not '/':
	textfile = textfile + '/'

print textfile


import pandas as pd
import os,time


def getopts(argv):
    opts = {}  # Empty dictionary to store key-value pairs.
    while argv:  # While there are arguments left to parse...
        if argv[0][0] == '-':  # Found a "-name value" pair.
            opts[argv[0]] = argv[1]  # Add key and value to the dictionary.
        argv = argv[1:]  # Reduce the argument list by copying it starting from index 1.
    return opts
def sort():

    from sys import argv
    myargs = getopts(argv)

    if '-i' in myargs:  # Example usage.
        textfile = os.path.abspath(myargs['-i'])
        split = textfile.split(os.path.sep)
        if split[0]=='C:' :
            split[0]='mnt'
            split[1]='mydata'
            join = os.path.join(*split)
            textfile = join
        if textfile[-1] is not '/':
            textfile = textfile + '/'
    else: textfile = '/mnt/mydata/kyuha/desktop/CP_Output/raw_images_1/'
    if '-o' in myargs:
        csvfile = myargs['-o']
    else: csvfile = '/mnt/mydata/kyuha/desktop/hi.csv'

    #cwd = os.path.abspath(os.path.dirname(__file__))
    #cwd = cwd.replace('sourcecode','')
    #cwd+textfile for normal script use in open func.
    #with open(textfile, 'r') as myfile:
    #    direc=myfile.read().strip()
    direc = textfile.strip()

    if direc[-1] is not '/':
        direc = direc + '/'

    from masksort import *
    masksort(direc)
    from UI_generator import *
    UI_generator(direc)
    ### below code means nothing, but needed for galaxy format ###
    d = {'dir':[direc]}
    df = pd.DataFrame(data=d)
    df.to_csv(csvfile,index=False, header=False)

    return('sort completed')



from copy import deepcopy
import numpy as np

def reg_bd3(bd0=None,bdr0=None):

    xc=np.sum(np.dot(bd0[1],abs(bd0[0]))) / np.sum(abs(bd0[0]))
    yc=np.sum(np.dot(bd0[0],abs(bd0[1]))) / np.sum(abs(bd0[1]))

    bd0=np.append([bd0[0]-yc], [bd0[1]-xc],axis=0)
    bd=deepcopy(bd0)
    bdr=deepcopy(bdr0)

    xc=np.sum(np.dot(bdr[1],abs(bdr[0]))) / np.sum(abs(bdr[0]))
    yc=np.sum(np.dot(bdr[0],abs(bdr[1]))) / np.sum(abs(bdr[1]))

    bdr=np.append([bdr[0] - yc],[bdr[1] - xc],axis=0)
    temp=deepcopy(bdr[1])
    bdr[1]=bdr[0]
    bdr[0]=temp
    temp=deepcopy(bd[1])
    bd[1]=bd[0]
    bd[0]=temp
    N=len(bd[0])
    costold=np.mean(sum(sum(np.power((bdr - bd),2))))
    bdout=deepcopy(bd)
    for k in range(1,N+1):
        idk=np.append(range(k,N+1),range(1,k))
        bdt = np.empty([len(idk),2])
        bdt[:]=np.nan
        for i in range(len(bd.transpose())):
            ind = int(idk[i]-1) 
            temp=bd.transpose()[ind]
            bdt[i] = temp
        temp=np.dot(bdr,bdt)
        u,_,v=np.linalg.svd(temp)
        v = v.T
        q=np.dot(v,u.transpose())
        bdtemp=np.dot(bdt,q)
        costnew=np.mean(sum(sum(np.power((bdr.transpose() - bdtemp),2))))
        if costnew < costold:
            bdout=deepcopy(bdtemp)
            costold=deepcopy(costnew)
        a= bdt[-1:0:-1]
        b= bdt[0]
        bdt = np.concatenate((bdt[-1:0:-1],[bdt[0]]),axis=0)
        temp=np.dot(bdr,bdt)
        u,_,v=np.linalg.svd(temp)
        v = v.T
        q=np.dot(v,u.T)
        bdtemp=np.dot(bdt,q)
        costnew=np.mean(sum(sum(np.power((bdr.T - bdtemp),2))))
        if costnew < costold:
            bdout=deepcopy(bdtemp)
            costold=deepcopy(costnew)
    regbd = deepcopy(bdout.T)
    regbd[:] = np.nan 
    regbd[0] = deepcopy(bdout.T[1])
    regbd[1] = deepcopy(bdout.T[0])
    ytot=costold.T
    return regbd, ytot
